# vim:ts=4
# ActivityLog.pm Perl module: Parse Nagios log file and extract status records
#
# Packages: Nagios::ActivityLog, Nagios::ActivityLog::Filter

use strict;
package Nagios::ActivityLog;
use Time::Local;

my $VERSION = "0.2";

# Status Log file fieldnames (after the first '[time] TYPE;'
#my( @hostfields ) = (
#	qw/host state last_check last_state_change acknowledged time_up/,
#	qw/time_down time_unreachable last_notification notification_count/,
#	qw/notification_enabled event_handler_enabled checks_enabled flap_enabled/,
#	qw/fail_predict_enabled perf_data flapping percent_state_change/,
#	qw/scheduled_downtime_depth text/
#);
#my( @servicefields ) = (
#	qw/host service state last_hard_state retry state_type last_check/,
#	qw/next_check check_type last_state_change checks_enabled passive_enabled/,
#	qw/event_handler_enabled acknowledged time_up time_unknown time_warning/,
#	qw/time_critical last_notification notification_count notification_enabled/,
#	qw/latency execution_time flap_enabled flapping fail_predict_enabled/,
#	qw/percent_state_change scheduled_downtime_depth perf_data/,
#	qw/service_obsess_enabled text/
#);
# Activity log fieldnames (after the first '[time] TYPE: host;')
my( @ahostfields ) = ( qw/state state_type state_count text/);
my( @aservicefields ) = ( qw/service state state_type state_count text/);
my( @ahostdownfields ) = ( qw/state text/);
my( @aservicedownfields ) = ( qw/service state text/);
# Globals
my $LOGSTART = -1;
my $ERROR = "";
sub error {
	return $ERROR;
}
# Constructor
# $log = Nagios::ActivityLog->new("/u01/nagios/etc/nagios.conf");
sub new {
	my $pkg = shift;
	my $conf = $_[0];
	my $t;
	my $n = {};
	my $first = 0;
	my $line;

	if(! -r $conf) { $ERROR = "File $conf cannot be opened"; return 0; }

	open CONF, "<$conf" or  return 0;
	while( <CONF> ) {
		chomp;
		/^log_file\s*=\s*(.*)/ and do { $$n{logfile} = $1; };
		/^log_archive_path\s*=\s*(.*)/ and do { $$n{archivedir} = $1; last; };
	}
	close CONF;

	if(!$$n{archivedir} or !$$n{logfile} ) {
		$ERROR = "Cannot correctly parse $conf"; return 0;
	}

	# We assume the latest logs are for NOW.  The earliest logs we can 
	# identify by getting the time of the first entry in the available files
	# Also, the list of files themselves.
	$$n{logfiles} = [ glob($$n{archivedir}.'/*.log'), $$n{logfile} ];
	foreach my $f ( @{$$n{logfiles}} ) {
		open LF, "<$f";
		$line = <LF>;
		if( $line =~ /^\[(\d+)\]/ ) {
			$first = $1 if(!$first or $1<$first);
		}
		close LF;
	}

#	$Nagios::ActivityLog::LOGSTART = $LOGSTART;
	$LOGSTART = $$n{start} = $first;

	bless $n, 'Nagios::ActivityLog';
	return $n;
}
sub DESTROY {}; # For the benefit of Autoload

sub logstart {
	my($me, $v) = @_;
	$LOGSTART = $v if($v);
	return $LOGSTART;
}

# check list membership
sub _inlist($$) {
	my( $a, $l ) = @_;

	if( !ref $l ) {
		return 1 if($a eq $l);
		return 0;
	}
	foreach ( @$l ) { return 1 if($a eq $_); }
	return 0;
}

# Fetch some log entries.
# @entries = $log->fetch(%filter) 
# $entry = { host, service, time, text }
# %filter = ( start, end, host(), service(), 
sub fetch {
	my $me = shift;
	my %filter;
	my( $file, $line, $ftime);
	my( @files );
	my( $start, $end ) = (0,0);
	my(@fields);
	my($t,$type,$ftype,$host);
	my(%entry, @entries);
	my(@ltime,$d,$m,$y);
	my($rs);
	my($dstart,$dend) = (0,0);
	my($ns) = 0;

	# Parameters
	if ( ref $_[0] eq 'Nagios::ActivityLog::Filter' ) {
		%filter = %{$_[0]};
	} elsif( ref $_[0] eq 'HASH' ) {
		%filter = %{$_[0]};
	} else { %filter = @_; }

	@entries = ();

	# default times
	$end = $filter{end} if($filter{end});
	$start = $filter{start} if($filter{start});
	$end = time if(!$end);
	$start = $end-(24*3600) if(!$start);
	$ns = 1 if($filter{noservices});
	delete $filter{start};
	delete $filter{end};
#	(@ltime) = localtime($start-(24*3600));
#	($d,$m,$y) = ($ltime[3],$ltime[4]+1,$ltime[5]+1900);
	delete $filter{initial_state} if(defined $filter{initial_state});
	delete $filter{noservices} if(defined $filter{noservices});
#	delete $filter{hosts} if(defined $filter{hosts});
#	delete $filter{services} if(defined $filter{services});

	# Identify which log files to parse
	# This should be improved for efficiency, also sorting...
#	@files = ( glob($me->{archivedir}.'/*.log'), $me->{logfile} );
#	@files = ( $me->{logfiles} );

	# Read files in turn, stop if > endtime
FILE: foreach $file ( @{$me->{logfiles}} ) {
		# the filename is the rotate date: hence, the oldest record in the
		# file.  So, if this is < start we dont need the file.
		# if filename-1day (first record) > end we dont need it either.
		if( $file =~ /nagios-(\d\d)-(\d\d)-(\d\d\d\d)/ ) {
			# do a next if the file date is after $end, or more than 1 day 
			# before $start
			$ftime =  timelocal(0,0,0,$2,$1-1,$3-1900);
			next if($ftime < $start);
			next if(($ftime-(24*3600)) > $end);
#			next if(($3 < $y) or (($3 == $y) and($1 < $m))
#				 or (($3 == $y) and ( $1 == $m ) and ($2 < $d)));
			$dend = $ftime if($ftime > $dend);
			$ftime = $ftime -(24*3600);
			$dstart = $ftime if($ftime<$dstart or !$dstart);
		}
		open LOG, "<$file" or next;
		#print "Scanning logfile $file<br>\n";
ENTRY:	while( $line = <LOG> ) {
			chomp $line;
			@fields = split /;/,$line;
			next if( $fields[0] !~ /^\[(\d+)\]\s+(\S.*\S)\s*:\s*(\S+)/ );
			($t,$type,$host) = ($1,$2,$3);
			next if($t < $start);
			last if($t > $end  );
#			next if($ftype and ($type ne $ftype));
			next if($type !~ /ALERT/ );
			shift @fields;

			# parse entry
			%entry = ( type=>$type, 'time'=>$t, host=>$host );
			if( $type eq 'HOST ALERT' ) {
				foreach (@ahostfields) { $entry{$_} = shift @fields; 
					last if(!@fields); }
			} elsif($type eq 'SERVICE ALERT') { # service
				next if($ns);
				foreach (@aservicefields) { $entry{$_} = shift @fields;
					last if(!@fields); }
			} elsif($type eq 'HOST DOWNTIME ALERT') {    # host downtime
				foreach (@ahostdownfields) { $entry{$_} = shift @fields;
					last if(!@fields); }
			} elsif($type eq 'SERVICE DOWNTIME ALERT') { # service downtime
				next if($ns);
				foreach (@aservicedownfields) { $entry{$_} = shift @fields;
					last if(!@fields); }
			} else { next; }

			# check against filter
			foreach ( keys %filter ) {
				next if(!$filter{$_}); # null means all
				next ENTRY if( !$entry{$_}
					or !Nagios::ActivityLog::_inlist($entry{$_}, $filter{$_}));
			}

			# store
			push @entries, {%entry};
		}
		close LOG;
	}
	$rs = \@entries;
	bless $rs, 'Nagios::ActivityLog::RecordSet';
	return $rs;
}

sub fetchxml {
	my $me = shift;
	my $recordset;
	my $xml;

	$recordset = $me->fetch(@_);
	$xml = $recordset->xml(@_);

	return $xml;
}

sub fetchsummary {
	my $me = shift;
	my $recordset;
	my $summary;

	$recordset = fetch($me,@_);
	$summary = $recordset->summary(@_);

	return $summary;
}


##############################################################
# Filter.  This object can be used to make a new filter.
package Nagios::ActivityLog::Filter;

sub new {
	my( $pkg ) = shift;
	my( %options ) = @_;
	my( $f ) = {};

	# Passed parameters
	%$f = %options;

	# Default parameters
	$f->{end} = time if(!$f->{end});
	$f->{start} = $f->{end}-(24*3600) if(!defined $f->{start});

	bless $f, 'Nagios::ActivityLog::Filter';
	return $f;
}
sub fetch($$) {
	my( $me ) = shift;
	my( $f ) = shift;
	my( $rv, $a );
	$a = Nagios::ActivityLog->new($f);
	$rv = $a->fetch($me);
	return $rv;
}
sub fetchxml($$) {
	my( $me ) = shift;
	my( $f ) = shift;
	my( $a );
	$a = Nagios::ActivityLog->new($f);
	return $a->fetchxml($me);
}
sub add($$@) {
	my( $me ) = shift;
	my( $key ) = shift;
	my( @values ) = @_;

	if($#values > 0) {
		$me->{$key} = [ @values ];
	} else {
		$me->{$key} = $values[0];
	}
	return $me;
}

package Nagios::ActivityLog::RecordSet;

# Convert to XML
sub xml {
	my ($me) = shift;
	my ($xml) = "";
	my (%filter);
	my ($entry, @foo);

	# Parameters
	if ( ref $_[0] eq 'Nagios::ActivityLog::Filter' ) {
		%filter = %{$_[0]};
	} elsif( ref $_[0] eq 'HASH' ) {
		%filter = %{$_[0]};
	} else { %filter = @_; }

	# Here, build up the XML
	$xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
	$xml .= "<!DOCTYPE NAGIOS PUBLIC \"-//University of Auckland//Nagios::ActivityLog.pm//EN//\" \"http://www.steveshipway.org/software\">";
	$xml .= "<NAGIOS SOURCE=\"Nagios::ActivityLog\" TIMESTAMP=".time.">\n";

	if(@_) { # we had a filter 
		$xml .= "<FILTER>\n";
		foreach (keys %filter) {
			$xml .= "<FIELD NAME=$_>";
			if(ref $filter{$_}) {
				$xml .= ''.(join ',',@{$filter{$_}});
			} else { $xml .= $filter{$_}; }
			$xml .= "</FIELD>\n";
		}
		$xml .= "</FILTER>\n";
	}

	$xml .= "<DATA>\n";
	foreach $entry ( @$me ) {
		$entry->{text} =~ s/</&lt;/g; $entry->{text} =~ s/>/&gt;/g;
		@foo = split " ",$entry->{type};
		$xml .= "<".$foo[0].$foo[1]." HOSTNAME=".$entry->{host};
		if( $entry->{type} =~ /^SERVICE/ ) {
			$xml .= " SERVICENAME=\"".$entry->{service}."\""
		}
		$xml .= " STATE=".$entry->{state};
		if( $entry->{type} eq 'HOST ALERT' 
			or $entry->{type} eq 'SERVICE ALERT' ) {
			$xml .=	" TYPE=\"".$entry->{state_type}."\""
				." COUNT=".$entry->{state_count};
		}
		$xml .= ">".$entry->{text}."</".$foo[0].$foo[1].">\n";
	}
	$xml .= "</DATA></NAGIOS>\n";
	return $xml;
}
sub csv {
	my ($me) = shift;
	my ($csv) = "";
	my (%filter);
	my ($entry, @foo);

	# Parameters
	if ( ref $_[0] eq 'Nagios::ActivityLog::Filter' ) {
		%filter = %{$_[0]};
	} elsif( ref $_[0] eq 'HASH' ) {
		%filter = %{$_[0]};
	} else { %filter = @_; }

	# header
	$csv = "type,host,service,state,state_type,state_count,text\n";
	# records
	foreach $entry ( @$me ) {
		$csv .= $entry->{type}.','.$entry->{host};
		if( $entry->{type} =~ /^SERVICE/ ) {
			$csv .= ",\"".$entry->{service}."\""
		} else { $csv .= ",\"\""; }
		$csv .= ",".$entry->{state};
		if( $entry->{type} eq 'HOST ALERT' 
			or $entry->{type} eq 'SERVICE ALERT' ) {
			$csv .=	",".$entry->{state_type}.",".$entry->{state_count};
		} else { $csv .= ",,"; }
		$csv .= ",\"".$entry->{text}."\"\n";
	}
	return $csv;
}
# Further filter the recordset
sub filter {
	my( $me ) = shift;
	my( %filter );
	my( $r, $rs, @entries );

	if ( ref $_[0] eq 'Nagios::ActivityLog::Filter' ) {
		%filter = %{$_[0]};
	} elsif( ref $_[0] eq 'HASH' ) {
		%filter = %{$_[0]};
	} else { %filter = @_; }

	$rs = [];
REC: foreach $r ( @$me ) {
		foreach ( keys %filter ) {
			next if(!$filter{$_}); # null means all
			next REC if( !$r->{$_}
				or !Nagios::ActivityLog::_inlist($r->{$_}, $filter{$_}));
		}
		push @$rs, $r;
	}

	bless $rs, 'Nagios::ActivityLog::RecordSet';
	return $rs;
}

# Optionally pass it options/filter
# Valid options:
#    initial_state
#    start end
#    services = [ 'host;service' ]
#    hosts = [ 'host' ]
sub _bytime { return ($a->{'time'} <=> $b->{'time'}); }
sub summary {
	my( $me ) = shift;
	my( $options );
	my( $summary ) = {};
	my( $stime ) = 0;
	my( $etime ) = 0;
	my( $k, $is, $dis, $hk );
	my( $timea, $timeb, $timediff, $oldstate, $newstate );
	my( $ttot, $window );
	my( $sk, $snewstate, $soldstate, $stimea, $stimeb );
	my( $ostime );

	if( ref $_[0] ) {
		$options = $_[0];
	} else {
		$options = { @_ };
	}

	$dis = ''; 
	$dis = uc $options->{initial_state} 
		if($options and $options->{initial_state});
	$etime = $options->{end} if($options and $options->{end});
	$stime = $options->{start} if($options and $options->{start});
	if($stime) {
		$ostime = $stime;
		$stime = $Nagios::ActivityLog::LOGSTART 
			if($Nagios::ActivityLog::LOGSTART > $stime);
	}

	# Predefinitions
	if( defined $options->{host} and ref $options->{host} eq 'ARRAY' ) {
		$ostime = $stime = $Nagios::ActivityLog::LOGSTART if(!$stime);
		foreach $k ( @{$options->{host}} ) {
			$summary->{$k} = { 
				last_change=>$stime, last_state=>($dis?$dis:'UP'),
				UP=>0, DOWN=>0, UNREACHABLE=>0, in_downtime=>0,
				type=>'HOST' };
		}
	}
	if( defined $options->{service} and ref $options->{service} eq 'ARRAY' ) {
		$ostime = $stime = $Nagios::ActivityLog::LOGSTART if(!$stime);
		foreach $sk ( @{$options->{service}} ) {
			$summary->{$sk} = { type=>'SERVICE',
				last_change=>$stime, last_state=>($dis?$dis:'OK'),
				OK=>0, WARNING=>0, CRITICAL=>0, UNKNOWN=>0, in_downtime=>0 };
			$k = $sk; $k =~ s/;.*//;
			$summary->{$k} = { type=>'HOST',
				last_change=>$stime, last_state=>($dis?$dis:'UP'),
				UP=>0, DOWN=>0, UNREACHABLE=>0, in_downtime=>0 }
					if(!defined $summary->{$k});
		}
	}

	# sort the entries, and process chronologically.
	foreach my $e ( sort _bytime @$me ) {
		last if($etime and ($e->{'time'} > $etime)); # after our window
		$ostime = $stime = $e->{'time'} if(!$stime); # define our window
		if( $e->{type} eq 'HOST ALERT' ) {
			$k = $e->{host};
			$summary->{$k} = { type=>'HOST',
				last_change=>$stime, last_state=>($dis?$dis:'UP'),
				UP=>0, DOWN=>0, UNREACHABLE=>0, in_downtime=>0 }
				if(!defined $summary->{$k});
			$newstate = $e->{state};
			$newstate .= ":S" if($summary->{$k}{in_downtime});
		} elsif( $e->{type} eq 'SERVICE ALERT' ) {
			$k = $e->{host}.';'.$e->{service};
			$summary->{$k} = { type=>'SERVICE',
				last_change=>$stime, last_state=>($dis?$dis:'OK'),
				OK=>0, WARNING=>0, CRITICAL=>0, UNKNOWN=>0, in_downtime=>0 }
				if(!defined $summary->{$k});
			$newstate = $e->{state};
			if($summary->{$k}{in_downtime}) {
				$newstate .= ":S" 
			} else {
				$hk = $k; $hk =~ s/;.*$//;
				if( defined $summary->{$hk}
					and $summary->{$hk}{in_downtime} ) {
					$newstate .= ':H';
					$summary->{$k}{host_in_downtime} = 1;
				}
			}
		} elsif( $e->{type} eq 'HOST DOWNTIME ALERT' ) {
			$k = $e->{host};
			$summary->{$k} = { type=>'HOST',
				last_change=>$stime, last_state=>($dis?$dis:'UP'),
				UP=>0, DOWN=>0, UNREACHABLE=>0, in_downtime=>0 }
				if(!defined $summary->{$k});
			$newstate = $summary->{$k}{last_state};
			if($e->{state} eq 'STARTED') {
				$newstate .= ':S' if($newstate !~ /:S$/) ;
				$summary->{$k}{in_downtime} = 1;
			} else {
				$newstate =~ s/:S$//;
				$summary->{$k}{in_downtime} = 0;
			}
			# Now, for each service of this host, we have to update its
			# status.
			foreach $sk ( keys %$summary ) {
				next if($sk !~ /^$k;/);
				next if( $summary->{$sk}{in_downtime} );
				# status change for service
				$soldstate = $summary->{$sk}{last_state};
				$stimea = $summary->{$sk}{last_change};
				$stimea = $stime if($stimea<$stime);
				$stimeb = $e->{'time'};
				$snewstate = $soldstate; $snewstate =~ s/:H$//;
				$snewstate .= ":H" if($summary->{$k}{in_downtime});
				$summary->{$sk}{host_in_downtime}=$summary->{$k}{in_downtime};	
				$summary->{$sk}{last_state} = $snewstate;
				$summary->{$sk}{last_change} = $stimeb;
				next if($stimeb<$stime); # outside our window
				$timediff = $stimeb - $stimea;
				$summary->{$sk}{$soldstate} = 0 
					if(!defined $summary->{$sk}{$soldstate});
				$summary->{$sk}{$soldstate} += $timediff;
			}
		} elsif( $e->{type} eq 'SERVICE DOWNTIME ALERT' ) {
			$k = $e->{host}.';'.$e->{service};
			$summary->{$k} = { type=>'SERVICE',
				last_change=>$stime, last_state=>($dis?$dis:'OK'),
				OK=>0, WARNING=>0, CRITICAL=>0, UNKNOWN=>0, 
				in_downtime=>0, host_in_downtime=>0 }
				if(!defined $summary->{$k});
			$newstate = $summary->{$k}{last_state};
			if($e->{state} eq 'STARTED') {
				$newstate =~ s/:[HS]$//;
				$newstate .= ':S';
				$summary->{$k}{in_downtime} = 1;
			} else {
				my($hk);
				$newstate =~ s/:S$//;
				$summary->{$k}{in_downtime} = 0;
				$hk = $k; $hk =~ s/;.*$//;
				if( defined $summary->{$hk}
					and $summary->{$hk}{in_downtime} ) {
					$newstate .= ':H';
					$summary->{$k}{host_in_downtime} = 1;
				}
			}
		} else { next; }
		$oldstate = $summary->{$k}{last_state};
		$summary->{$k}{last_state} = $newstate;
		$timea = $summary->{$k}{last_change};
		$timeb = $e->{'time'};
		$timea = $stime if($timea < $stime);
		$timediff = $timeb - $timea;
		$summary->{$k}{last_change} = $timeb;
		next if($timediff < 0 or $timeb < $stime); # before our window
		$summary->{$k}{$oldstate} = 0 if(!defined $summary->{$k}{$oldstate});
		$summary->{$k}{$oldstate} += $timediff;
	}
	$etime = $timeb if(!$etime);

	# now finish them all off;
	$window = $etime - $stime;
	foreach $k ( keys %$summary ) {
		$timea = $summary->{$k}{last_change};
		$timea = $stime if($timea < $stime);
		$timediff = $etime - $timea;
		$oldstate = $summary->{$k}{last_state};
		$summary->{$k}->{$oldstate} = 0 if(!defined $summary->{$k}->{$oldstate});
		$summary->{$k}{$oldstate} += $timediff;
		delete $summary->{$k}{last_change};
		delete $summary->{$k}{last_state};
		delete $summary->{$k}{in_downtime};
		delete $summary->{$k}{host_in_downtime};
		$ttot = 0;
		foreach ( keys %{$summary->{$k}} ) { $ttot += $summary->{$k}{$_} if($_ ne 'type'); }
		$summary->{$k}->{MISSING} = ($window - $ttot)+($stime - $ostime);
	}

	bless $summary, 'Nagios::ActivityLog::Summary';
	return $summary;
}

# Required
1;

__DATA__
=head 1 NAME

Nagios::ActivityLog -- Parse and process the Nagios Activity Log files

=head1 SYNOPSIS

	use Nagios::ActivityLog;
	$log = Nagios::ActivityLog->new('nagios.cfg');
	$records = $log->fetch($filter);
	$xml = $log->fetchxml($filter);
	$summary = $log->fetchsummary($filter);
	$lasterror = $log->error();

	$filter = {
		fieldname=>'value',
		fieldname=>['value','value','value'],
		start=>time,
		end=>time,
		initial_state=>'OK',         (for summary ONLY)
		host=>['hosta','hostb',...], 
		service=>['svca','svcb',...]  
	}
	$filter = Nagios::ActivityLog::Filter->new(fieldname=>'value'...);
	$filter = Nagios::ActivityLog::Filter->new(fieldname=>['value','value']...);
	$records = $filter->fetch('nagios.cfg');
	$xml = $filter->fetchxml('nagios.cfg');
	$filter->add(fieldname=>'value'...);
	$filter->add(fieldname=>['value','value']...);
	
	$records = [
		{ fieldname=>'value', fieldname=>'value' ... },
		{ fieldname=>'value', fieldname=>'value' ... }
	];
	$records = $records->filter($filter);
	$xml = $records->xml();
	$csv = $records->csv();
	$summary = $records->summary($filter);

	$summary = {
		'host' => { type=>'HOST', UP=>pct, 'DOWN:S'=>pct, DOWN=>pct, 
			UNKNOWN=>pct, MISSING=>pct }
		'host;service' => { type=>'SERVICE', OK=>pct, MISSING=>pct,
			WARNING:S=>pct, WARNING:H=>pct, WARNING=>pct,
			CRITICAL:S=>pct, CRITICAL:H=>pct, CRITICAL=>pct,
			UNKNOWN:S=>pct, UNKNOWN:H=>pct, UNKNOWN=>pct
		}
	}
	$summary = $records->summary(%options);
	$summary = $records->summary($filter);

=head1 LIMITATIONS

This only processes entries for hosts and services states, including
scheduled and unscheduled downtimes.

Initial states are assumed as OK/UP.  If there are available extra records,
then these are used to determine initial states.  So, you can extract a
recordset for one day earlier in the past, then use the summary method with
a different filter parameter to statr later.

=head1 EXAMPLES

use Nagios::ActivityLog; # Steve's module for .log files

my($CFGFILE) = "/u01/nagios/etc/nagios.cfg";

@hosts = qw/foo bar baz/;

print "Creating Log object and Config object\n";
$log = Nagios::ActivityLog->new(Filename=>$CFGFILE);

print "Getting XML logs for one whole week for [$hostgroup]\n";
$filter = Nagios::ActivityLog::Filter->new(end=>time, start=>(time-(7*24*3600)), host=>[@hosts]);
$recordset = $log->fetch($filter);
$xml = $recordset->xml($filter);
open XML,">output.xml";
print XML $xml;
close XML;

$summary = $recordset->summary($filter);
foreach my $k ( keys %$summary ) {
	print "$k: ";
	foreach ( keys %{$summary->{$k}} ) {
		print "$_=".$summary->{$k}->{$_}." ";
	}
	print "\n";
}

print "Retrieving last 24hr logs (no filter)\n";
$entries = $log->fetch();

print "Retrieving yesterday's logs (TIME FILTER)\n";
$entries = $log->fetch(end=>(time-(24*3600)));

print "Retrieving one week's host logs\n";
$entries = $log->fetch(end=>time, start=>(time-(7*24*3600)), 
                       type=>['HOST ALERT','HOST DOWNTIME ALERT']);

print "Retrieving one days logs for mail servers, critical only\n";
$entries = $log->fetch(host=>[qw/groucho chico harpo zeppo/], 
                       state=>'CRITICAL' );

=head1 HISTORY

Created for use with the Nagios package by Steve Shipway in July 2004.  This
is expected to be used in conjunction with the existing Nagios::Config packages.

For updates and support, visit http://www.steveshipway.org/software/
Support forum on http://www.steveshipway.org/forum/
